##############################################################################
# $Id: compose.py,v 1.2 2004/04/21 13:24:04 andrey_kiselev Exp $
#
# Project:  OpenEV
# Purpose:  Interactive tool to compose/decompose image bands.
# Author:   Andrey Kiselev, dron@remotesensing.org
#	    Iscander Latypov
#
###############################################################################
# Copyright (c) 2004, American Museum of Natural History. All rights reserved.
# This software is based upon work supported by NASA under award
# number NAG5-12333
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
###############################################################################
# 
#  $Log: compose.py,v $
#  Revision 1.2  2004/04/21 13:24:04  andrey_kiselev
#  Check whether we have a raster layer on input.
#
#  Revision 1.1  2004/03/09 11:01:08  andrey_kiselev
#  New.
#
#
#

from gtk import *

import gview
import string
import gvutils
import gviewapp
import Numeric

import gdal
from gdalconst import *
import gdalnumeric

########################################################################
def get_raster_size(_layer):
    w =  _layer.get_parent().get_dataset().GetRasterBand(1).XSize
    h =  _layer.get_parent().get_dataset().GetRasterBand(1).YSize
    return (w,h)

#########################################################################
def get_list_of_bands_as_dict():
    """Returns dictionary of the bands of the opened layers. The key 
    of dictionary is the GtkListItem object produced from the name of
    layer+band number, members of dictionary are view-object (index = 0),
    layer-object (index = 1), band-number and name"""

    layer = gview.app.sel_manager.get_active_layer()
    if layer is None:
	return {}
    _size = get_raster_size(layer)
    dict = {}
    for curview in gview.app.view_manager.view_list:
        for curlayer in curview.viewarea.list_layers():
    	    curname = curlayer.get_name()
	    cursize = get_raster_size(curlayer)
	    if cursize == _size:
		num_bands = curlayer.get_parent().get_dataset().RasterCount
		for i in range(1,num_bands+1):
	    	    curband = curname + '.band['+ str(i) + ']'
		    dict[GtkListItem(curband)] = (curview,curlayer,i,curband) 
    if dict is None:
	return None
    return dict

############################################################################
#
#
############################################################################
class ComposeTool(gviewapp.Tool_GViewApp):
    
    def __init__(self,app=None):
        gviewapp.Tool_GViewApp.__init__(self,app)
        self.init_menu()

    def launch_dialog(self,*args):
	try:
	    self.win = ComposeDialog()
	except:
            gvutils.error("Please select a raster layer.")
	    return
    	self.win.show()

    def init_menu(self):
        self.menu_entries.set_entry("Image/Compose...", 2, self.launch_dialog)

############################################################################
#
#
############################################################################
class ComposeDialog(GtkWindow):

    def __init__(self,app=None):
	rlayer = gview.app.sel_manager.get_active_layer()
	if rlayer is None:
            raise TypeError
        try:
    	    gdal_dataset = rlayer.get_data().get_dataset()
        except:
    	    raise TypeError
	
	GtkWindow.__init__(self)
	self.set_title('Compose')
	
	gui_OK = self.create_gui()
	if gui_OK is FALSE:
	    return None
	else:
	    GtkWindow.show_all(self)
	
    def close(self,*args):
	self.hide()
	self.visibility_flag = 0
	return TRUE

    def create_gui(self):

	self.input_bands = get_list_of_bands_as_dict()
	self.output_bands = {}
	
	box1 = GtkVBox(spacing=10)
	box1.set_border_width(10)
        self.add(box1)
        box1.show()

	frame1 = GtkFrame('Select image bands to compose')
	frame1.show()
        box1.pack_start(frame1, expand=FALSE)

	box2 = GtkVBox(spacing=10)
	box2.set_border_width(10)
	frame1.add(box2)
	box2.show()

	box3 = GtkHBox(spacing=10)
	box3.set_border_width(10)
	box2.pack_start(box3,expand=FALSE)
	box3.show()

### source ##############################################################
	source_win = GtkScrolledWindow()
	source_win.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
	source_win.set_usize(300,200)
	box3.pack_start(source_win)
	source_win.show()

	source_list = GtkList()
	source_list.set_selection_mode(SELECTION_MULTIPLE)
	source_list.set_selection_mode(SELECTION_BROWSE)
	source_win.add_with_viewport(source_list)
	source_list.show()

	source_list.append_items(self.input_bands.keys())

### control buttons ######################################################
	btn_box = GtkVBox(spacing=10)
	btn_box.set_border_width(10)
	box3.pack_start(btn_box,expand=FALSE)
	btn_box.show()

	def dest_add(_button,*args):
	    for i in source_list.get_selection():
		list_item = GtkListItem(self.input_bands[i][3])
		dest_list.add(list_item)
		list_item.show()
		dest_list.select_child(dest_list.children()[-1])
		self.output_bands[list_item] = self.input_bands[i]

	def dest_del(_button,*args):
	    selection = dest_list.get_selection()
	    dest_list.remove_items(selection)
	    for i in selection:
		del self.output_bands[i]
		i.destroy()
	    rest = dest_list.children()
	    if len(rest) > 0:
		dest_list.select_child(dest_list.children()[-1])

	add_btn = GtkButton("Add->")
        add_btn.connect("clicked", dest_add)
	btn_box.pack_start(add_btn)
	add_btn.show()
	del_btn = GtkButton("<-Remove")
        del_btn.connect("clicked", dest_del)
	btn_box.pack_start(del_btn)
	del_btn.show()

##  destination ##########################################################
	dest_win = GtkScrolledWindow()
	dest_win.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
	dest_win.set_usize(300,200)
	box3.pack_start(dest_win)
	dest_win.show()

	dest_list = GtkList()
	dest_list.set_selection_mode(SELECTION_MULTIPLE)
	dest_list.set_selection_mode(SELECTION_BROWSE)
	dest_win.add_with_viewport(dest_list)
	dest_list.show()

#####OUT TYPES#########################################################
	box_types = GtkHBox(spacing=10)
	box2.pack_start(box_types, expand=FALSE)
	box_types.show()
	
	types_label  = GtkLabel('Out Image Type:')
	types_label.set_alignment(0, 0.5)
	box_types.pack_start(types_label, expand=FALSE)
	
	self.types_list = []
	i = GDT_Byte
	while i < GDT_TypeCount:
	    self.types_list.append(gdal.GetDataTypeName(i))
	    i += 1
	self.types = gvutils.GvOptionMenu(self.types_list)
	box_types.pack_start(self.types)
	self.types.show()
	
	# Open in the current view or create the new one
	self.switch_new_view = GtkCheckButton("Create new view")
	box1.pack_start(self.switch_new_view)
	self.switch_new_view.show()

####BUTTONS#############################################################
	separator = GtkHSeparator()
	box1.pack_start(separator, expand=FALSE)

	box3 = GtkHBox(spacing=10)
        box1.pack_start(box3, expand=FALSE)
        box3.show()

        execute_btn = GtkButton("Ok")
        execute_btn.connect("clicked", self.compose)
	box3.pack_start(execute_btn)
        
        close_btn = GtkButton("Close")
        close_btn.connect("clicked", self.close)
        box3.pack_start(close_btn)
	
        return TRUE

    def compose(self,*args):
	if len(self.output_bands) <= 0:
	    return
	dest_bands = []
	for i in self.output_bands.values():
	    ds = i[1].get_parent().get_dataset()
	    dest_bands.append(ds.GetRasterBand(i[2]))

	type = gdal.GetDataTypeByName(self.types_list[self.types.get_history()])
	# FIXME: perform proper scaling here
	numtype = gdalnumeric.GDALTypeCodeToNumericTypeCode(type)
	res = Numeric.array(map(lambda i: i.ReadAsArray().astype(numtype), dest_bands), numtype)
	res_ds = gdalnumeric.OpenArray(res, self.output_bands.values()[0][1].get_parent().get_dataset())
	if self.switch_new_view.get_active():
	    gview.app.new_view()
	gview.app.open_gdal_dataset(res_ds)
   
TOOL_LIST = ['ComposeTool']

